import time
import logging
import WebFrame as WF
from WebFrame import handler
from WebFrame import template
try:
    from xml.etree import cElementTree as ET
except ImportError:
    try:
        from elementtree import cElementTree as ET
    except ImportError:
        from elementtree import ElementTree as ET


# Set config options
WF.set_opt("file.default_encoding", "iso-8859-1")
WF.set_opt("http.default_encoding", "iso-8859-1")
WF.set_opt("template.allow_pathinfo", True)

# Set handlers
def h_json(ctx, js):
    logging.info("I gots me some json: %s", str(js))
    return { "my":{ "a" : u"jj" }, "other":js }
handler.register_json("ANY", "/manual/json", ".*", h_json, "@public","@public")

def h_xml(ctx, xml):
    logging.info("I gots me some xml: %s", str(xml))
    x2 =  ET.Element("yo")
    x2.append(ET.Element("eep"))
    if xml:
        x2.append(xml)
    return "application/smegma+xml", x2
handler.register_xml("ANY", "/manual/xml", ".*", h_xml, "@public","@public")


def h_plain(ctx):
    ctx.resp.write("PathInfo: %s\n" % str(ctx.req.path_info))
    ctx.resp.write("Query: %s\n" % str(ctx.req.params))
    return ctx.resp.stop_processing()
handler.register_plain("ANY", "/manual/plain", ".*", h_plain,"@public","@deny")

def h_cookie(ctx):
    ctx.resp.set_cookie("cookie", "smells great", time.time()+600)
    ctx.resp.write("Got cookies: %s\n" % str(ctx.req.cookies))
    return ctx.resp.stop_processing()
handler.register_plain("ANY", "/manual/cookie", ".*", h_cookie,
                       "@public", "@deny")

def h_session(ctx):
    ctx.resp.write("This time session was: %s\n" % str(ctx.sess))
    if "poke" in ctx.sess:
        ctx.sess["poke"] += 1
    else:
        ctx.sess["poke"] = 1
    ctx.resp.write("Now it is: %s\n" % str(ctx.sess))
    return ctx.resp.stop_processing()
handler.register_plain("ANY", "/manual/session", ".*", h_session,
                       "@public","@deny")


# Database setup
WF.set_opt("db.host", "localhost")
WF.set_opt("db.user", "test")
WF.set_opt("db.name", "test")
WF.set_opt("db.pass", "test")
def sql_injection(ctx):
    """BIG WARNING!! THIS IS A SECURITY RISK!!!
    This functon is for demo only!!!!"""
    db = WF.open_db()
    ctx.resp.write("Asking for [%s]\n" % ctx.req.path_info)
    cr = db.simple(ctx.req.path_info)
    for i in cr:
        ctx.resp.write(unicode(i)+"\n")
    ctx.resp.write("All done")
handler.register_plain("GET", "/sql", ".*", sql_injection,"@public","@public")


# Register some template helpers
template.register_value("the_var", 42)
template.register_func("funky", lambda ctx, x: "funky "+str(x),
                       "@loggedin", "@loggedin")

# Register a DB handler, that returns an iterator to the template
# iterators are better since they use less resources.
# Fortunately the database result cursor can provide an iterator =)
# WARNING: Make sure to call iter() on the object, as passing it directly
#          to the template can expose stuff the template shouldn't touch!
def DB_list_sessions(ctx):
    dbh = WF.open_db()
    cr = dbh.simple("SELECT * FROM Session")
    return iter(cr)
template.register_func("DB_list_sessions", DB_list_sessions,
                       "@loggedin", "@loggedin")


# Query parameter handlers demo.
# This is called when a query contains ACTION:demo=whatever
# It stops processing of the template, but could let it continue
# if it returned ctx.resp.cont() instead
def qhandler(ctx, action):
    ctx.resp.write("Qhandler triggered for %s\n" % action)
    ctx.resp.write("Parameters are: %s\n" % ctx.req.params)
    return ctx.resp.stop_processing()
handler.register_query("demo", qhandler, "@public", "@public")
